searchData={"items":[{"type":"module","title":"Vx","doc":"Vx is a schema validation library.","ref":"Vx.html"},{"type":"function","title":"Vx.valid?/2","doc":"Checks if a value is valid against a given schema.","ref":"Vx.html#valid?/2"},{"type":"function","title":"Examples - Vx.valid?/2","doc":"iex> Vx.valid?(Vx.String.t(), \"foo\")\n    true\n\n    iex> Vx.valid?(Vx.String.t(), 123)\n    false","ref":"Vx.html#valid?/2-examples"},{"type":"function","title":"Vx.validate/2","doc":"Validates a value against a given schema.","ref":"Vx.html#validate/2"},{"type":"function","title":"Examples - Vx.validate/2","doc":"iex> Vx.validate(Vx.String.t(), \"foo\")\n    :ok\n\n    iex> Vx.validate(Vx.String.t(), 123)\n    {:error, %Vx.Error{message: \"must be a string\", schema: Vx.String.t(), value: 123}}","ref":"Vx.html#validate/2-examples"},{"type":"function","title":"Vx.validate!/2","doc":"Validates a value against a given schema. Raises on error.","ref":"Vx.html#validate!/2"},{"type":"function","title":"Examples - Vx.validate!/2","doc":"iex> Vx.validate!(Vx.String.t(), \"foo\")\n    :ok\n\n    iex> Vx.validate!(Vx.String.t(), 123)\n    ** (Vx.Error) must be a string","ref":"Vx.html#validate!/2-examples"},{"type":"type","title":"Vx.schema/0","doc":"A Vx schema is anything that implements the `Vx.Validatable` protocol.","ref":"Vx.html#t:schema/0"},{"type":"module","title":"Vx.Any","doc":"The Any type.","ref":"Vx.Any.html"},{"type":"function","title":"Vx.Any.t/0","doc":"Builds a new type that matches anything.","ref":"Vx.Any.html#t/0"},{"type":"function","title":"Examples - Vx.Any.t/0","doc":"iex> Vx.Any.t() |> Vx.validate!(\"foo\")\n    :ok","ref":"Vx.Any.html#t/0-examples"},{"type":"opaque","title":"Vx.Any.t/0","doc":"The any type.","ref":"Vx.Any.html#t:t/0"},{"type":"module","title":"Vx.Atom","doc":"The Atom type.","ref":"Vx.Atom.html"},{"type":"function","title":"Vx.Atom.t/0","doc":"Builds a new Atom type.","ref":"Vx.Atom.html#t/0"},{"type":"function","title":"Examples - Vx.Atom.t/0","doc":"iex> Vx.Atom.t() |> Vx.validate!(:foo)\n    :ok\n\n    iex> Vx.Atom.t() |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be an atom\n\nAs `nil`, booleans and module names are also atoms, all of these are totally\nvalid as well:\n\n    iex> Vx.Atom.t() |> Vx.validate!(nil)\n    :ok\n\n    iex> Vx.Atom.t() |> Vx.validate!(true)\n    :ok\n\n    iex> Vx.Atom.t() |> Vx.validate!(Address)\n    :ok","ref":"Vx.Atom.html#t/0-examples"},{"type":"opaque","title":"Vx.Atom.t/0","doc":"The atom type.","ref":"Vx.Atom.html#t:t/0"},{"type":"module","title":"Vx.Boolean","doc":"The Boolean type.","ref":"Vx.Boolean.html"},{"type":"function","title":"Vx.Boolean.t/0","doc":"Builds a new Boolean type.","ref":"Vx.Boolean.html#t/0"},{"type":"function","title":"Examples - Vx.Boolean.t/0","doc":"iex> Vx.Boolean.t() |> Vx.validate!(true)\n    :ok\n\n    iex> Vx.Boolean.t() |> Vx.validate!(false)\n    :ok\n\n    iex> Vx.Boolean.t() |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be a boolean","ref":"Vx.Boolean.html#t/0-examples"},{"type":"opaque","title":"Vx.Boolean.t/0","doc":"The boolean type.","ref":"Vx.Boolean.html#t:t/0"},{"type":"module","title":"Vx.Enum","doc":"The Enum type.","ref":"Vx.Enum.html"},{"type":"function","title":"Vx.Enum.t/1","doc":"Builds a new Enum type.","ref":"Vx.Enum.html#t/1"},{"type":"function","title":"Examples - Vx.Enum.t/1","doc":"iex> Vx.Enum.t([:foo, :bar]) |> Vx.validate!(:foo)\n    :ok\n\n    iex> Vx.Enum.t([:foo, :bar]) |> Vx.validate!(:baz)\n    ** (Vx.Error) must be one of :foo, :bar","ref":"Vx.Enum.html#t/1-examples"},{"type":"type","title":"Vx.Enum.t/0","doc":"","ref":"Vx.Enum.html#t:t/0"},{"type":"opaque","title":"Vx.Enum.t/1","doc":"","ref":"Vx.Enum.html#t:t/1"},{"type":"module","title":"Vx.Float","doc":"The Float type.","ref":"Vx.Float.html"},{"type":"function","title":"Vx.Float.integer/1","doc":"Requires the float to have no decimal places.","ref":"Vx.Float.html#integer/1"},{"type":"function","title":"Examples - Vx.Float.integer/1","doc":"iex> Vx.Float.integer() |> Vx.validate!(123.0)\n    :ok\n\n    iex> Vx.Float.integer() |> Vx.validate!(123.4)\n    ** (Vx.Error) must have no decimal places\n\n    iex> Vx.Float.integer() |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be a float","ref":"Vx.Float.html#integer/1-examples"},{"type":"function","title":"Vx.Float.t/0","doc":"Builds a new Float type.","ref":"Vx.Float.html#t/0"},{"type":"function","title":"Examples - Vx.Float.t/0","doc":"iex> Vx.Float.t() |> Vx.validate!(1.0)\n    :ok\n\n    iex> Vx.Float.t() |> Vx.validate!(1)\n    ** (Vx.Error) must be a float\n\n    iex> Vx.Float.t() |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be a float","ref":"Vx.Float.html#t/0-examples"},{"type":"opaque","title":"Vx.Float.t/0","doc":"The float type.","ref":"Vx.Float.html#t:t/0"},{"type":"module","title":"Vx.Integer","doc":"The Integer type.","ref":"Vx.Integer.html"},{"type":"function","title":"Vx.Integer.t/0","doc":"Builds a new Integer type.","ref":"Vx.Integer.html#t/0"},{"type":"function","title":"Examples - Vx.Integer.t/0","doc":"iex> Vx.Integer.t() |> Vx.validate!(1)\n    :ok\n\n    iex> Vx.Integer.t() |> Vx.validate!(1.0)\n    ** (Vx.Error) must be an integer\n\n    iex> Vx.Integer.t() |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be an integer","ref":"Vx.Integer.html#t/0-examples"},{"type":"opaque","title":"Vx.Integer.t/0","doc":"The integer type.","ref":"Vx.Integer.html#t:t/0"},{"type":"module","title":"Vx.List","doc":"The List type.","ref":"Vx.List.html"},{"type":"function","title":"Vx.List.max_size/2","doc":"Requires the list to have a maximum size.","ref":"Vx.List.html#max_size/2"},{"type":"function","title":"Vx.List.min_size/2","doc":"Requires the list to have a minimum size.","ref":"Vx.List.html#min_size/2"},{"type":"function","title":"Vx.List.non_empty/1","doc":"Requires the list to be non-empty.","ref":"Vx.List.html#non_empty/1"},{"type":"function","title":"Examples - Vx.List.non_empty/1","doc":"iex> Vx.List.non_empty() |> Vx.validate!([1, 2, 3])\n    :ok\n\n    iex> Vx.List.non_empty() |> Vx.validate!([])\n    ** (Vx.Error) must not be empty","ref":"Vx.List.html#non_empty/1-examples"},{"type":"function","title":"Vx.List.shape/2","doc":"Requires the list to match the given shape.","ref":"Vx.List.html#shape/2"},{"type":"function","title":"Examples - Vx.List.shape/2","doc":"iex> Vx.List.shape([Vx.Number.t(), Vx.String.t()]) |> Vx.validate!([123, \"foo\"])\n    :ok\n\n    iex> Vx.List.shape([Vx.Number.t(), Vx.String.t()]) |> Vx.validate!([123])\n    ** (Vx.Error) must match [number, string]\n    - element 1 is missing\n\n    iex> Vx.List.shape([Vx.Number.t(), Vx.String.t()]) |> Vx.validate!([123, :foo])\n    ** (Vx.Error) must match [number, string]\n    - element 1: must be a string","ref":"Vx.List.html#shape/2-examples"},{"type":"function","title":"Vx.List.size/2","doc":"","ref":"Vx.List.html#size/2"},{"type":"function","title":"Vx.List.t/0","doc":"Builds a new List type.","ref":"Vx.List.html#t/0"},{"type":"function","title":"Examples - Vx.List.t/0","doc":"iex> Vx.List.t() |> Vx.validate!([1, 2, 3])\n    :ok\n\n    iex> Vx.List.t() |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be a list","ref":"Vx.List.html#t/0-examples"},{"type":"function","title":"Vx.List.t/1","doc":"Builds a new List type with the given inner type.","ref":"Vx.List.html#t/1"},{"type":"function","title":"Examples - Vx.List.t/1","doc":"iex> Vx.List.t(Vx.Number.t()) |> Vx.validate!([1, 2, 3])\n    :ok\n\n    iex> Vx.List.t(Vx.String.t()) |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be a list\n\n    iex> Vx.List.t(Vx.String.t()) |> Vx.validate!([\"foo\", 2, \"bar\"])\n    ** (Vx.Error) must be a list \n    - element 1: must be a string","ref":"Vx.List.html#t/1-examples"},{"type":"opaque","title":"Vx.List.t/0","doc":"The list type.","ref":"Vx.List.html#t:t/0"},{"type":"module","title":"Vx.Literal","doc":"The Literal type.","ref":"Vx.Literal.html"},{"type":"function","title":"Vx.Literal.t/1","doc":"Builds a new Literal type from a value.","ref":"Vx.Literal.html#t/1"},{"type":"function","title":"Examples - Vx.Literal.t/1","doc":"iex> Vx.Literal.t(:foo) |> Vx.validate!(:foo)\n    :ok\n\n    iex> Vx.Literal.t(:foo) |> Vx.validate!(:bar)\n    ** (Vx.Error) must be :foo\n\nNote that everything not being a type (to be precise anything not implementing\nthe `Vx.Validatable` protocol) is automatically considered a literal. So this\nis equivalent to the previous example:\n\n    iex> :foo |> Vx.validate!(:foo)\n    :ok\n\n    iex> :foo |> Vx.validate!(:bar)\n    ** (Vx.Error) must be :foo","ref":"Vx.Literal.html#t/1-examples"},{"type":"type","title":"Vx.Literal.t/0","doc":"","ref":"Vx.Literal.html#t:t/0"},{"type":"opaque","title":"Vx.Literal.t/1","doc":"","ref":"Vx.Literal.html#t:t/1"},{"type":"module","title":"Vx.Map","doc":"The Map type.","ref":"Vx.Map.html"},{"type":"function","title":"Vx.Map.shape/2","doc":"Checks the shape of the map.","ref":"Vx.Map.html#shape/2"},{"type":"function","title":"Examples - Vx.Map.shape/2","doc":"iex> schema = Vx.Map.shape(%{a: Vx.String.t(), b: Vx.Number.t()})\n    ...> Vx.validate!(schema, %{a: \"foo\", b: 123})\n    :ok\n\n    iex> schema = Vx.Map.shape(%{a: Vx.String.t(), b: Vx.Number.t()})\n    ...> Vx.validate!(schema, %{a: \"foo\"})\n    ** (Vx.Error) must have key(s) :b\n\n    iex> schema = Vx.Map.shape(%{a: Vx.String.t(), b: Vx.Number.t()})\n    ...> Vx.validate!(schema, %{a: \"foo\", b: \"bar\"})\n    ** (Vx.Error) does not match shape\n    - key :b: must be a number\n\nIt is also possible to mark certain keys as optional.\n\n    iex> schema = Vx.Map.shape(%{\n    ...>   :a => Vx.String.t(),\n    ...>   Vx.Optional.t(:b) => Vx.Number.t()\n    ...> })\n    ...> Vx.validate!(schema, %{a: \"foo\"})\n    :ok\n\n    iex> schema = Vx.Map.shape(%{\n    ...>   a: Vx.String.t(),\n    ...>   b: Vx.Optional.t(Vx.Number.t())\n    ...> })\n    ...> Vx.validate!(schema, %{a: \"foo\"})\n    :ok\n\n    iex> schema = Vx.Map.shape(%{\n    ...>   :a => Vx.String.t(),\n    ...>   Vx.Optional.t(:b) => Vx.Number.t()\n    ...> })\n    ...> Vx.validate!(schema, %{a: \"foo\", b: \"bar\"})\n    ** (Vx.Error) does not match shape\n    - key :b: must be a number","ref":"Vx.Map.html#shape/2-examples"},{"type":"function","title":"Vx.Map.size/2","doc":"Checks the size of the map.","ref":"Vx.Map.html#size/2"},{"type":"function","title":"Examples - Vx.Map.size/2","doc":"iex> Vx.Map.size(0) |> Vx.validate!(%{})\n    :ok\n\n    iex> Vx.Map.size(1) |> Vx.validate!(%{a: \"foo\", b: 123})\n    ** (Vx.Error) must have a size of 1","ref":"Vx.Map.html#size/2-examples"},{"type":"function","title":"Vx.Map.t/0","doc":"Builds a new Map type that matches any map.","ref":"Vx.Map.html#t/0"},{"type":"function","title":"Examples - Vx.Map.t/0","doc":"iex> Vx.Map.t() |> Vx.validate!(%{})\n    :ok\n\n    iex> Vx.Map.t() |> Vx.validate!(%{a: \"foo\", b: 123})\n    :ok\n\n    iex> Vx.Map.t() |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be a map","ref":"Vx.Map.html#t/0-examples"},{"type":"function","title":"Vx.Map.t/2","doc":"Builds a new Map type with a specific key and value type.","ref":"Vx.Map.html#t/2"},{"type":"function","title":"Examples - Vx.Map.t/2","doc":"iex> Vx.Map.t(Vx.String.t(), Vx.Number.t()) |> Vx.validate!(%{})\n    :ok\n\n    iex> schema = Vx.Map.t(Vx.Atom.t(), Vx.Number.t())\n    ...> Vx.validate!(schema, %{a: 123, b: 234.5})\n    :ok\n\n    iex> Vx.Map.t(Vx.Atom.t(), Vx.Number.t()) |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be a map \n\n    iex> Vx.Map.t(Vx.Atom.t(), Vx.Number.t()) |> Vx.validate!(%{foo: \"bar\"})\n    ** (Vx.Error) must be a map \n    - value of element :foo: must be a number","ref":"Vx.Map.html#t/2-examples"},{"type":"opaque","title":"Vx.Map.t/0","doc":"The map type.","ref":"Vx.Map.html#t:t/0"},{"type":"module","title":"Vx.Number","doc":"The Integer type provides validators for integers.","ref":"Vx.Number.html"},{"type":"function","title":"Vx.Number.between/3","doc":"Requires the number to be within the given range.","ref":"Vx.Number.html#between/3"},{"type":"function","title":"Vx.Number.gt/2","doc":"Requires the number to be greater than the given value.","ref":"Vx.Number.html#gt/2"},{"type":"function","title":"Vx.Number.gteq/2","doc":"Requires the number to be greater than or equal to the given value.","ref":"Vx.Number.html#gteq/2"},{"type":"function","title":"Vx.Number.integer/1","doc":"Requires the number to have no decimal places.","ref":"Vx.Number.html#integer/1"},{"type":"function","title":"Examples - Vx.Number.integer/1","doc":"iex> Vx.Number.integer() |> Vx.validate!(123)\n    :ok\n\n    iex> Vx.Number.integer() |> Vx.validate!(123.0)\n    :ok\n\n    iex> Vx.Number.integer() |> Vx.validate!(123.4)\n    ** (Vx.Error) must have no decimal places\n\n    iex> Vx.Number.integer() |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be a number","ref":"Vx.Number.html#integer/1-examples"},{"type":"function","title":"Vx.Number.lt/2","doc":"Requires the number to be less than the given value.","ref":"Vx.Number.html#lt/2"},{"type":"function","title":"Vx.Number.lteq/2","doc":"Requires the number to be less than or equal to the given value.","ref":"Vx.Number.html#lteq/2"},{"type":"function","title":"Vx.Number.negative/1","doc":"Requires the number to be negative.","ref":"Vx.Number.html#negative/1"},{"type":"function","title":"Vx.Number.positive/1","doc":"Requires the number to be positive.","ref":"Vx.Number.html#positive/1"},{"type":"function","title":"Vx.Number.range/2","doc":"Requires the number to be in the given range.","ref":"Vx.Number.html#range/2"},{"type":"function","title":"Example - Vx.Number.range/2","doc":"iex> Vx.Number.range(1..10) |> Vx.validate!(5)\n    :ok\n\n    iex> Vx.Number.range(1..10) |> Vx.validate!(11)\n    ** (Vx.Error) must be in 1..10","ref":"Vx.Number.html#range/2-example"},{"type":"function","title":"Vx.Number.t/0","doc":"Builds a new Number type.","ref":"Vx.Number.html#t/0"},{"type":"function","title":"Examples - Vx.Number.t/0","doc":"iex> Vx.Number.t() |> Vx.validate!(123)\n    :ok\n\n    iex> Vx.Number.t() |> Vx.validate!(123.4)\n    :ok\n\n    iex> Vx.Number.t() |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must be a number","ref":"Vx.Number.html#t/0-examples"},{"type":"type","title":"Vx.Number.numeric/0","doc":"","ref":"Vx.Number.html#t:numeric/0"},{"type":"opaque","title":"Vx.Number.t/0","doc":"The number type.","ref":"Vx.Number.html#t:t/0"},{"type":"module","title":"Vx.String","doc":"The String type.","ref":"Vx.String.html"},{"type":"function","title":"Vx.String.format/2","doc":"Requires a string to match the given regex.","ref":"Vx.String.html#format/2"},{"type":"function","title":"Examples - Vx.String.format/2","doc":"iex> Vx.String.t() |> Vx.String.format(~r/\\d+/) |> Vx.validate!(\"123\")\n    :ok\n\n    iex> Vx.String.t() |> Vx.String.format(~r/\\d+/) |> Vx.validate!(\"foo\")\n    ** (Vx.Error) must match expected format","ref":"Vx.String.html#format/2-examples"},{"type":"function","title":"Vx.String.max_length/2","doc":"Requires a string to be at most `length` characters long.","ref":"Vx.String.html#max_length/2"},{"type":"function","title":"Examples - Vx.String.max_length/2","doc":"iex> Vx.String.t() |> Vx.String.max_length(3) |> Vx.validate!(\"foo\")\n    :ok\n\n    iex> Vx.String.t() |> Vx.String.max_length(3) |> Vx.validate!(\"fo\")\n    :ok\n\n    iex> Vx.String.t() |> Vx.String.max_length(3) |> Vx.validate!(\"fooo\")\n    ** (Vx.Error) must be at most 3 characters","ref":"Vx.String.html#max_length/2-examples"},{"type":"function","title":"Vx.String.min_length/2","doc":"Requires a string to be at least `length` characters long.","ref":"Vx.String.html#min_length/2"},{"type":"function","title":"Examples - Vx.String.min_length/2","doc":"iex> Vx.String.t() |> Vx.String.min_length(3) |> Vx.validate!(\"foo\")\n    :ok\n\n    iex> Vx.String.t() |> Vx.String.min_length(3) |> Vx.validate!(\"foob\")\n    :ok\n\n    iex> Vx.String.t() |> Vx.String.min_length(3) |> Vx.validate!(\"fo\")\n    ** (Vx.Error) must be at least 3 characters","ref":"Vx.String.html#min_length/2-examples"},{"type":"function","title":"Vx.String.non_empty/1","doc":"Requires a string to be non-empty.","ref":"Vx.String.html#non_empty/1"},{"type":"function","title":"Examples - Vx.String.non_empty/1","doc":"iex> Vx.String.t() |> Vx.String.non_empty() |> Vx.validate!(\"foo\")\n    :ok\n\n    iex> Vx.String.t() |> Vx.String.non_empty() |> Vx.validate!(\"   \")\n    :ok\n\n    iex> Vx.String.t() |> Vx.String.non_empty() |> Vx.validate!(\"\")\n    ** (Vx.Error) must not be empty","ref":"Vx.String.html#non_empty/1-examples"},{"type":"function","title":"Vx.String.present/1","doc":"Requires a string to be non-empty after stripping leading and trailing\nwhitespace.","ref":"Vx.String.html#present/1"},{"type":"function","title":"Examples - Vx.String.present/1","doc":"iex> Vx.String.t() |> Vx.String.present() |> Vx.validate!(\"foo\")\n    :ok\n\n    iex> Vx.String.t() |> Vx.String.present() |> Vx.validate!(\"\")\n    ** (Vx.Error) must be present\n\n    iex> Vx.String.t() |> Vx.String.present() |> Vx.validate!(\"   \")\n    ** (Vx.Error) must be present","ref":"Vx.String.html#present/1-examples"},{"type":"function","title":"Vx.String.t/0","doc":"Builds a new String type.","ref":"Vx.String.html#t/0"},{"type":"function","title":"Examples - Vx.String.t/0","doc":"iex> Vx.String.t() |> Vx.validate!(\"foo\")\n    :ok\n\n    iex> Vx.String.t() |> Vx.validate!(123)\n    ** (Vx.Error) must be a string","ref":"Vx.String.html#t/0-examples"},{"type":"opaque","title":"Vx.String.t/0","doc":"The string type.","ref":"Vx.String.html#t:t/0"},{"type":"module","title":"Vx.Struct","doc":"The Struct type.","ref":"Vx.Struct.html"},{"type":"function","title":"Vx.Struct.t/0","doc":"Builds a new Struct type matching any type of struct.","ref":"Vx.Struct.html#t/0"},{"type":"function","title":"Examples - Vx.Struct.t/0","doc":"iex> Vx.Struct.t() |> Vx.validate!(%Address{})\n    :ok\n\n    iex> Vx.Struct.t() |> Vx.validate!(%{})\n    ** (Vx.Error) must be a struct","ref":"Vx.Struct.html#t/0-examples"},{"type":"function","title":"Vx.Struct.t/1","doc":"Builds a new Struct type matching a specific type of struct.","ref":"Vx.Struct.html#t/1"},{"type":"function","title":"Examples - Vx.Struct.t/1","doc":"iex> Vx.Struct.t(Address) |> Vx.validate!(%Address{})\n    :ok\n\n    iex> Vx.Struct.t(Address) |> Vx.validate!(%{})\n    ** (Vx.Error) must be a struct of type Address\n\n    iex> Vx.Struct.t(Address) |> Vx.validate!(%Country{})\n    ** (Vx.Error) must be a struct of type Address","ref":"Vx.Struct.html#t/1-examples"},{"type":"opaque","title":"Vx.Struct.t/0","doc":"The struct type.","ref":"Vx.Struct.html#t:t/0"},{"type":"module","title":"Vx.Tuple","doc":"The Tuple type.","ref":"Vx.Tuple.html"},{"type":"function","title":"Vx.Tuple.shape/2","doc":"Requires a tuple to match a specific shape.","ref":"Vx.Tuple.html#shape/2"},{"type":"function","title":"Examples - Vx.Tuple.shape/2","doc":"iex> Vx.Tuple.shape({:foo, :bar}) |> Vx.validate!({:foo, :bar})\n    :ok\n\n    iex> Vx.Tuple.shape({Vx.Atom.t(), Vx.String.t()}) |> Vx.validate!({:ok, \"result\"})\n    :ok\n\n    iex> Vx.Tuple.shape({Vx.Atom.t(), Vx.String.t()}) |> Vx.validate!({:ok, 123})\n    ** (Vx.Error) must match {atom, string}\n    - element 1: must be a string","ref":"Vx.Tuple.html#shape/2-examples"},{"type":"function","title":"Vx.Tuple.size/2","doc":"Requires a tuple to have a specific size.","ref":"Vx.Tuple.html#size/2"},{"type":"function","title":"Examples - Vx.Tuple.size/2","doc":"iex> Vx.Tuple.t() |> Vx.Tuple.size(2) |> Vx.validate!({:foo, :bar})\n    :ok\n\n    iex> Vx.Tuple.t() |> Vx.Tuple.size(2) |> Vx.validate!({:foo})\n    ** (Vx.Error) must have a size of 2","ref":"Vx.Tuple.html#size/2-examples"},{"type":"function","title":"Vx.Tuple.t/0","doc":"Builds a new Tuple type.","ref":"Vx.Tuple.html#t/0"},{"type":"function","title":"Examples - Vx.Tuple.t/0","doc":"iex> Vx.Tuple.t() |> Vx.validate!({:foo, :bar})\n    :ok\n\n    iex> Vx.Tuple.t() |> Vx.validate!(123)\n    ** (Vx.Error) must be a tuple","ref":"Vx.Tuple.html#t/0-examples"},{"type":"opaque","title":"Vx.Tuple.t/0","doc":"The tuple type.","ref":"Vx.Tuple.html#t:t/0"},{"type":"module","title":"Vx.Intersect","doc":"The Intersect type combines multiple types into a single type, validating\nwhether all of them are valid.","ref":"Vx.Intersect.html"},{"type":"function","title":"Vx.Intersect.t/1","doc":"Builds a new Intersect type.","ref":"Vx.Intersect.html#t/1"},{"type":"function","title":"Examples - Vx.Intersect.t/1","doc":"iex> Vx.Intersect.t([Vx.Integer.t(), Vx.Number.t()]) |> Vx.validate!(123)\n    :ok\n\n    iex> Vx.Intersect.t([Vx.Integer.t(), Vx.Number.t()]) |> Vx.validate!(12.3)\n    ** (Vx.Error) must be all of (integer & number)","ref":"Vx.Intersect.html#t/1-examples"},{"type":"type","title":"Vx.Intersect.t/0","doc":"","ref":"Vx.Intersect.html#t:t/0"},{"type":"opaque","title":"Vx.Intersect.t/1","doc":"","ref":"Vx.Intersect.html#t:t/1"},{"type":"module","title":"Vx.Union","doc":"The Union type combines multiple types into a single type, validating\nwhether any of them is valid.","ref":"Vx.Union.html"},{"type":"function","title":"Vx.Union.t/1","doc":"Builds a new Union type.\n\n# Examples\n\n    iex> Vx.Union.t([Vx.Integer.t(), Vx.String.t()]) |> Vx.validate!(123)\n    :ok\n\n    iex> Vx.Union.t([Vx.Integer.t(), Vx.String.t()]) |> Vx.validate!(:foo)\n    ** (Vx.Error) must be any of (integer | string)","ref":"Vx.Union.html#t/1"},{"type":"type","title":"Vx.Union.t/0","doc":"","ref":"Vx.Union.html#t:t/0"},{"type":"opaque","title":"Vx.Union.t/1","doc":"","ref":"Vx.Union.html#t:t/1"},{"type":"module","title":"Vx.Not","doc":"The Not type negates the given type or value.","ref":"Vx.Not.html"},{"type":"function","title":"Vx.Not.t/1","doc":"Builds a new type negating the passed one.","ref":"Vx.Not.html#t/1"},{"type":"function","title":"Examples - Vx.Not.t/1","doc":"iex> Vx.Not.t(Vx.Integer.t()) |> Vx.validate!(\"foo\")\n    :ok\n\n    iex> Vx.Not.t(Vx.Integer.t()) |> Vx.validate!(123)\n    ** (Vx.Error) must not be integer","ref":"Vx.Not.html#t/1-examples"},{"type":"type","title":"Vx.Not.t/0","doc":"","ref":"Vx.Not.html#t:t/0"},{"type":"opaque","title":"Vx.Not.t/1","doc":"","ref":"Vx.Not.html#t:t/1"},{"type":"module","title":"Vx.Nullable","doc":"The Nullable type modifies a type or value to allow `nil` as a valid value.","ref":"Vx.Nullable.html"},{"type":"function","title":"Vx.Nullable.t/1","doc":"Builds a new type that makes the passed type nullable.","ref":"Vx.Nullable.html#t/1"},{"type":"function","title":"Examples - Vx.Nullable.t/1","doc":"iex> Vx.Nullable.t(Vx.String.t()) |> Vx.validate!(\"foo\")\n    :ok\n\n    iex> Vx.Nullable.t(Vx.String.t()) |> Vx.validate!(nil)\n    :ok\n\n    iex> Vx.Nullable.t(Vx.String.t()) |> Vx.validate!(123)\n    ** (Vx.Error) must be (string | nil)","ref":"Vx.Nullable.html#t/1-examples"},{"type":"type","title":"Vx.Nullable.t/0","doc":"","ref":"Vx.Nullable.html#t:t/0"},{"type":"opaque","title":"Vx.Nullable.t/1","doc":"","ref":"Vx.Nullable.html#t:t/1"},{"type":"module","title":"Vx.Optional","doc":"The Optional type provides validators for optional keys in a map. When used in\nother places, it behaves like `Vx.Nullable`.","ref":"Vx.Optional.html"},{"type":"function","title":"Vx.Optional.t/1","doc":"Builds a new type that makes the passed type optional.","ref":"Vx.Optional.html#t/1"},{"type":"function","title":"Examples - Vx.Optional.t/1","doc":"iex> Vx.Optional.t(Vx.String.t()) |> Vx.validate!(\"foo\")\n    :ok\n\n    iex> Vx.Optional.t(Vx.String.t()) |> Vx.validate!(nil)\n    :ok\n\n    iex> Vx.Optional.t(Vx.String.t()) |> Vx.validate!(123)\n    ** (Vx.Error) must be string?\n\nIn most cases the behavior is the same as using `Vx.Nullable.t/1`.\n\nBesides that, you can use `Vx.Optional.t/1` to mark map keys as optional.\n\n    iex> schema = Vx.Map.shape(%{\n    ...>   :a => Vx.String.t(),\n    ...>   Vx.Optional.t(:b) => Vx.Number.t()\n    ...> })\n    ...> Vx.validate!(schema, %{a: \"foo\"})\n    :ok\n\n    iex> schema = Vx.Map.shape(%{\n    ...>   :a => Vx.String.t(),\n    ...>   Vx.Optional.t(:b) => Vx.Number.t()\n    ...> })\n    ...> Vx.validate!(schema, %{a: \"foo\", b: \"bar\"})\n    ** (Vx.Error) does not match shape\n    - key :b: must be a number","ref":"Vx.Optional.html#t/1-examples"},{"type":"type","title":"Vx.Optional.t/0","doc":"","ref":"Vx.Optional.html#t:t/0"},{"type":"opaque","title":"Vx.Optional.t/1","doc":"","ref":"Vx.Optional.html#t:t/1"},{"type":"module","title":"Vx.Match","doc":"The Match helper checks whether a value matches a pattern. As this relies on\nmacros, you need to `require Vx.Match` before using it.","ref":"Vx.Match.html"},{"type":"macro","title":"Vx.Match.t/1","doc":"Creates a new type that matches a pattern.","ref":"Vx.Match.html#t/1"},{"type":"macro","title":"Examples - Vx.Match.t/1","doc":"iex> require Vx.Match\n    ...> schema = Vx.Match.t(%{a: _, b: _})\n    ...> Vx.validate!(schema, %{a: 1, b: 2})\n    :ok\n\n    iex> require Vx.Match\n    ...> schema = Vx.Match.t(%{a: _, b: _})\n    ...> Vx.validate!(schema, %{a: 1, c: 2})\n    ** (Vx.Error) must match %{a: _, b: _}","ref":"Vx.Match.html#t/1-examples"},{"type":"module","title":"Vx.Validator","doc":"A helper that can be used to validate values using a custom function.","ref":"Vx.Validator.html"},{"type":"function","title":"Vx.Validator.t/1","doc":"Builds a new helper type that can be used to validate values using a custom\nfunction.","ref":"Vx.Validator.html#t/1"},{"type":"function","title":"Examples - Vx.Validator.t/1","doc":"iex> Vx.Validator.t(&(&1 == \"foo\")) |> Vx.validate!(\"foo\")\n    :ok\n\n    iex> Vx.Validator.t(&(&1 == \"foo\")) |> Vx.validate!(\"bar\")\n    ** (Vx.Error) is invalid","ref":"Vx.Validator.html#t/1-examples"},{"type":"type","title":"Vx.Validator.fun/0","doc":"","ref":"Vx.Validator.html#t:fun/0"},{"type":"type","title":"Vx.Validator.t/0","doc":"","ref":"Vx.Validator.html#t:t/0"},{"type":"protocol","title":"Vx.Inspectable","doc":"The Inspectable protocol allows converting types to a string representation.","ref":"Vx.Inspectable.html"},{"type":"function","title":"Vx.Inspectable.inspect/1","doc":"","ref":"Vx.Inspectable.html#inspect/1"},{"type":"type","title":"Vx.Inspectable.t/0","doc":"All the types that implement this protocol.","ref":"Vx.Inspectable.html#t:t/0"},{"type":"protocol","title":"Vx.Validatable","doc":"The Validatable protocol that is implemented by all types of the Vx type\nsystem. As a fallback, any value not having an implementation for the\n`Vx.Validatable` protocol is considered a `Vx.Literal` when passed to\n`validate/2`.","ref":"Vx.Validatable.html"},{"type":"function","title":"Vx.Validatable.validate/2","doc":"Validates a value against a given validatable.","ref":"Vx.Validatable.html#validate/2"},{"type":"type","title":"Vx.Validatable.t/0","doc":"All the types that implement this protocol.","ref":"Vx.Validatable.html#t:t/0"},{"type":"module","title":"Vx.Constraint","doc":"A constraint for a `Vx.Type`. You can add a constraint to a type using\n`Vx.Type.constrain/3` and `Vx.Type.constrain/4`.","ref":"Vx.Constraint.html"},{"type":"type","title":"Vx.Constraint.fun/0","doc":"","ref":"Vx.Constraint.html#t:fun/0"},{"type":"type","title":"Vx.Constraint.t/0","doc":"","ref":"Vx.Constraint.html#t:t/0"},{"type":"module","title":"Vx.Type","doc":"A basic type that implements the `Vx.Validatable` protocol and allows adding\nconstraints to the type definition.","ref":"Vx.Type.html"},{"type":"macro","title":"Vx.Type.__using__/1","doc":"Turns a module into a custom type. It automatically implements the\n`Vx.Validatable` and `Vx.Inspectable` protocols for the newly created type and\nallows easily chaining constraints on your type.","ref":"Vx.Type.html#__using__/1"},{"type":"macro","title":"Example - Vx.Type.__using__/1","doc":"defmodule MyType do\n      use Vx.Type, :my_type\n\n      # The type constructor\n      @spec t() :: t\n      def t do\n        new(fn value ->\n          if my_type_is_valid?(value) do\n            :ok\n          else\n            {:error, \"my type is not valid\"}\n          end\n        end)\n      end\n\n      # A function that adds a constraint to a type\n      @spec foo(t) :: t\n      def foo(%__MODULE__{} = type \\\\ t()) do\n        constrain(schema, :my_constraint, fn value ->\n          if value < 123, do: :ok, else: {:error, \"must be less than 123\"}\n        end)\n      end\n\n      # ...\n    end","ref":"Vx.Type.html#__using__/1-example"},{"type":"function","title":"Vx.Type.constrain/4","doc":"Adds a constraint to the type.","ref":"Vx.Type.html#constrain/4"},{"type":"function","title":"Vx.Type.constraints/1","doc":"Returns the constraints of the type.","ref":"Vx.Type.html#constraints/1"},{"type":"function","title":"Vx.Type.constraints/2","doc":"Returns the constraints of the type with the given name.","ref":"Vx.Type.html#constraints/2"},{"type":"function","title":"Vx.Type.new/3","doc":"Creates a new type.","ref":"Vx.Type.html#new/3"},{"type":"type","title":"Vx.Type.custom/2","doc":"","ref":"Vx.Type.html#t:custom/2"},{"type":"type","title":"Vx.Type.fun/0","doc":"","ref":"Vx.Type.html#t:fun/0"},{"type":"type","title":"Vx.Type.t/0","doc":"","ref":"Vx.Type.html#t:t/0"},{"type":"type","title":"Vx.Type.t/1","doc":"","ref":"Vx.Type.html#t:t/1"},{"type":"exception","title":"Vx.Error","doc":"An error that is returned or raised when validation fails.","ref":"Vx.Error.html"},{"type":"type","title":"Vx.Error.t/0","doc":"","ref":"Vx.Error.html#t:t/0"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2024 Tobias Casper\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the \"Software\"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Readme","doc":"# Vx\n\n[![Build](https://github.com/tlux/vx/actions/workflows/elixir.yml/badge.svg)](https://github.com/tlux/vx/actions/workflows/elixir.yml)\n[![Coverage Status](https://coveralls.io/repos/github/tlux/vx/badge.svg?branch=main)](https://coveralls.io/github/tlux/vx?branch=main)\n[![Module Version](https://img.shields.io/hexpm/v/vx.svg)](https://hex.pm/packages/vx)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/vx/)\n[![License](https://img.shields.io/hexpm/l/vx.svg)](https://github.com/tlux/vx/blob/main/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/tlux/vx.svg)](https://github.com/tlux/vx/commits/main)\n\nThe Elixir schema validator.","ref":"readme.html"},{"type":"extras","title":"Installation - Readme","doc":"The package can be installed by adding `vx` to your list of dependencies in\n`mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:vx, \"~> 0.4.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Readme","doc":"With Vx, you have the capability to define schemata for validating complex data\neffortlessly.\n\nFirst, you need to define your schema.\n\n```elixir\nschema = Vx.String.t()\n```\n\nAfter that, you can call `Vx.validate/2` or `Vx.validate!/2` to check if a given\nvalues matches:\n\n```elixir\nVx.validate(schema, \"foo\")\n# :ok\n```\n\nWhen the value does not match, an error is returned (or raised, respectively),\nindicating the specific issue.\n\n```elixir\nVx.validate(schema, 123)\n# {:error, %Vx.Error{...}}\n```\n\n```elixir\nVx.validate!(schema, 123)\n# ** (Vx.Error) must be a string\n```\n\nAdditional constraints can be added to certain types by piping everything\ntogether:\n\n```elixir\nVx.Number.t()\n|> Vx.Number.gteq(3)\n|> Vx.Number.lt(7)\n```\n\nYou can combine multiple types and constraints to validate more complex\nschemata:\n\n```elixir\nVx.Map.shape(%{\n  \"name\" => Vx.String.t(),\n  \"age\" => Vx.Number.t(),\n  \"hobbies\" =>\n    Vx.List.t(Vx.String.present())\n    |> Vx.List.non_empty(),\n  \"type\" => Vx.Enum.t([\"user\", \"admin\"]),\n  \"addresses\" => Vx.List.t(Vx.Struct.t(Address))\n})\n```","ref":"readme.html#usage"},{"type":"extras","title":"Docs - Readme","doc":"Take a look at the [documentation](https://hexdocs.pm/vx) to find out available\ntypes and options.","ref":"readme.html#docs"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}